@@ -10,9 +10,8 @@ mpi.intercomm.merge(.intercomm,1,.comm)
 mpi.comm.set.errhandler(.comm)
 mpi.comm.disconnect(.intercomm)
 mpi.hostinfo(.comm)
-repeat {
+repeat 
     try(eval(mpi.bcast.cmd(rank=0,comm=.comm),envir=sys.parent()),TRUE)
-}
 print("Done")
 mpi.comm.disconnect(.comm)
 #mpi.abort(0)
